#version 300 es

precision highp float;

uniform float u_Time;
uniform vec2 u_Offset;
uniform float u_Speed;
uniform float u_Scale; // масштабирование градиента [0..1) - отдаление, (1..] - приближение
uniform int u_Complexity;

uniform float u_HUEOffset;
uniform float u_SatFactor;
uniform float u_LightFactor;

uniform sampler2D u_Mask;
uniform int u_UseMask;

const float color_intensity = 0.8;

in vec2 v_GCoord;

layout(location = 0) out vec4 o_FragColor;

bool is_in_range(vec3 range, vec3 col, float delta) {
	return(((range.r-delta < col.r) && (col.r < range.r+delta)) && ((range.g-delta < col.g) && (col.g < range.g+delta)) && ((range.b-delta < col.b) && (col.b < range.b+delta)));
}

vec3 range_clamp (vec3 col, vec3 col_center, float delta) {
	vec3 new_col;
	if (!is_in_range(vec3(col_center.r, col_center.g, col_center.b), col, delta)) {
		new_col.r = clamp(col.r, col_center.r-delta, col_center.r+delta);
		new_col.g = clamp(col.g, col_center.g-delta, col_center.g+delta);
		new_col.b = clamp(col.b, col_center.b-delta, col_center.b+delta);
	} else {
		new_col = col;
	}
	return new_col;
}

vec3 getcolor (vec3 col)
{
	float delta = 0.2;

	vec3 col1 = range_clamp(col, vec3(0.459, 0.224, 0.914), delta); //фиолетовый
	vec3 col2 = range_clamp(col, vec3(0.251, 0.867, 0.902), delta); //голубой
	vec3 col3 = range_clamp(col, vec3(0.219, 0.224, 0.713), delta); //этот случайный, но выглядит красиво; вообще должен быть вот этот: #38f6b6, т.е. vec3(0.219, 0.964, 0.713) -- салатовый
	vec3 col4 = range_clamp(col, vec3(0.953, 0.408, 0.247), delta); //оранжевый (циан?)
	vec3 col5 = range_clamp(col, vec3(0.894, 0.239, 0.713), delta); //розовый

	col = col1/5.0+col2/5.0+col3/5.0+col4/5.0+col5/5.0;

	return col;
}


vec3 rgb2hsv(vec3 c)
{
    vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
    vec4 p = mix(vec4(c.bg, K.wz), vec4(c.gb, K.xy), step(c.b, c.g));
    vec4 q = mix(vec4(p.xyw, c.r), vec4(c.r, p.yzx), step(p.x, c.r));

    float d = q.x - min(q.w, q.y);
    float e = 1.0e-10;
    return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
}


vec3 hsv2rgb(vec3 c)
{
    vec4 K = vec4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
    vec3 p = abs(fract(c.xxx + K.xyz) * 6.0 - K.www);
    return c.z * mix(K.xxx, clamp(p - K.xxx, 0.0, 1.0), c.y);
}

void main()
{
    vec2 p = (2.0 * v_GCoord - vec2(2.0)) / (2.0 * u_Scale);

    p += u_Offset;

    float t = u_Time / u_Speed;

    for(int i=1;i<u_Complexity;i++)
    {
        vec2 newp=p + t*0.001;
        newp.x+=0.6/float(i)*sin(float(i)*p.y+t+0.3*float(i)) + 2.22795;// + mouse.y/mouse_factor+mouse_offset;
        newp.y+=0.6/float(i)*sin(float(i)*(p.x)+t+0.3*float(i+10)) + 40.7;// - mouse.x/mouse_factor+mouse_offset;
        p=newp;
    }
    vec3 col=vec3(color_intensity*sin(3.0*p.x)+color_intensity,color_intensity*sin(3.0*p.y)+color_intensity,color_intensity*sin(p.x+p.y)+color_intensity);


    col = getcolor(col);

    col = rgb2hsv(col);
    col.x = mod(col.x + u_HUEOffset, 1.0); //hue
    col.y = col.y*1.5 * u_SatFactor; //saturation
    col.z = col.z*1.5 * u_LightFactor; //value
    col = hsv2rgb(col);

    float a = 1.0;

    if (u_UseMask == 1) {
        vec2 texCoords = vec2(v_GCoord.x + 0.5, 1.0 - (v_GCoord.y + 0.5));
        a = texture(u_Mask, texCoords).r;
    }

    o_FragColor=vec4(col, 1.0) * a;
}