#version 300 es

precision highp float;

uniform int u_GType; // 0 - линейный, 1 - радиальный
uniform vec2 u_GStart; // начало градиента, -1..1 для линейного, 0..1 для радиального
uniform vec2 u_GEnd; // конец градиента, -1..1 для линейного, 0..1 для радиального
uniform vec4 u_Color1; // начальный цвет
uniform vec4 u_Color2; // конечный цвет

uniform float u_XF;
uniform float u_YF;
uniform sampler2D u_Mask;

in highp vec2 v_GCoord; // координаты градиента, -0.5..0.5 (у контура фигуры может выходить за этот интервал)

layout(location = 0) out highp vec4 o_FragColor;

float map(float value, float inMin, float inMax, float outMin, float outMax) {
    return (value - inMin) * (outMax - outMin) / (inMax - inMin) + outMin;
}

void main() {
    vec4 color = vec4(1.0);
    vec2 texCoords = vec2(v_GCoord.x + 0.5, 1.0 - (v_GCoord.y + 0.5));
    float maskAlpha = texture(u_Mask, texCoords).r;

    // линейный
    if (u_GType == 0) {
        float a = atan(-u_GEnd.y + u_GStart.y, u_GEnd.x - u_GStart.x);
        float srx = u_GStart.x * cos(a) - u_GStart.y * sin(a);
        float erx = u_GEnd.x * cos(a) - u_GEnd.y * sin(a);
        float xlr = v_GCoord.x * u_XF * cos(a) + v_GCoord.y * u_YF * sin(a);

        color = mix(u_Color1, u_Color2, map(xlr, srx, erx, 0.0, 1.0));
    }

    // радиальный
    if (u_GType == 1) {
        float d = sqrt(pow(v_GCoord.x * u_XF, 2.0) + pow(v_GCoord.y * u_YF, 2.0)); // центр (0, 0)

        color = mix(u_Color1, u_Color2, map(d, u_GStart.y, u_GEnd.y, 0.0, 1.0));
    }

    o_FragColor = color * maskAlpha;
}
