#version 300 es

precision highp float;

uniform sampler2D u_ldMask;
uniform sampler2D u_Texture;
in vec2 v_TexCoord;

layout(location = 0) out vec4 o_FragColor;

float map(float value, float inMin, float inMax, float outMin, float outMax) {
    return (value - inMin) * (outMax - outMin) / (inMax - inMin) + outMin;
}

vec3 rgb2hsv(vec3 c) {
    vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
    vec4 p = mix(vec4(c.bg, K.wz), vec4(c.gb, K.xy), step(c.b, c.g));
    vec4 q = mix(vec4(p.xyw, c.r), vec4(c.r, p.yzx), step(p.x, c.r));

    float d = q.x - min(q.w, q.y);
    float e = 1.0e-10;
    return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
}

vec3 hsv2rgb(vec3 c) {
    vec4 K = vec4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
    vec3 p = abs(fract(c.xxx + K.xyz) * 6.0 - K.www);
    return c.z * mix(K.xxx, clamp(p - K.xxx, 0.0, 1.0), c.y);
}

void main() {
    vec2 ld = texture(u_ldMask, v_TexCoord).rg;
    vec4 tex = texture(u_Texture, v_TexCoord);
    vec3 color = rgb2hsv(tex.rgb);
    float v = 1.0;

    if (ld.r > 0.0) {
        v = 1.0 - ld.r;
    } else {
        v = map(ld.g, 0.0, 1.0, 1.0, 2.0);
    }

    color.z *= v;

    o_FragColor = vec4(hsv2rgb(color), tex.a);
}