#version 300 es

precision highp float;

in highp vec2 v_TexCoord;
uniform int u_Width;
uniform int u_Height;
uniform int u_PixelSize;
uniform float u_WindPowerX;
uniform float u_WindPowerY;
uniform int u_Type;
uniform sampler2D u_Noise;

uniform sampler2D u_Texture;

layout(location = 0) out vec4 o_FragColor;

float rand(float x, float y) {
    return fract(sin(dot(vec2(x, y), vec2(12.9898, 78.233))) * 43758.5453);
}

vec4 blur5(vec2 direction, vec2 size, vec4 curColor) {
    vec4 color = vec4(0.0);
    vec2 off1 = vec2(1.3333333333333333) * direction;

    color += curColor * 0.29411764705882354;
    color += texture(u_Texture, v_TexCoord + (off1 / size)) * 0.35294117647058826;
    color += texture(u_Texture, v_TexCoord + 2. * (off1 / size)) * 0.35294117647058826;

    return color;
}

void main() {
    float COUNT = 50.;
    float MAX_COOF = 0.15;
    float COOF_SMALL_SHIFT = 0.1;
    float SHIFT = 3.;
    float SHIFT_PER_PIXEL_SIZE_COOF = 0.0005;

    vec4 curColor = texture(u_Texture, v_TexCoord);
    vec4 color1 = curColor;
    vec4 color2 = texture(u_Texture, v_TexCoord / 2.);

    float windPowerX = u_WindPowerX * color1.r;
    float windPowerY = u_WindPowerY * color2.r;

     vec2 coord1 = v_TexCoord;
     vec2 coord2 = v_TexCoord;

     if (u_Type == 1) {
         coord2.x += windPowerX / float(u_Width);
         coord2.y += windPowerY / float(u_Height);
     }

     int pixelNumX = int((coord2.x * float(u_Width)) / float(u_PixelSize));
     int pixelNumY = int((coord2.y * float(u_Height)) / float(u_PixelSize));

     float coofXB;
     float coofYB;

     if(u_Type == 0) {
                 coofXB = (SHIFT_PER_PIXEL_SIZE_COOF * float(u_PixelSize) + SHIFT);
                 coofYB = (SHIFT_PER_PIXEL_SIZE_COOF * float(u_PixelSize) + SHIFT);
     } else if (u_Type == 1) {
                    float first = floor((coord1.x * float(u_Width)) / (float(u_PixelSize) / COUNT));
                    float second = floor((coord1.y * float(u_Height)) / (float(u_PixelSize) / COUNT));
                    coofYB = rand(first, 1.);
                    coofXB = rand(second, 1.);
     }


    vec2 currentPixel = vec2(float(pixelNumX) / (float(u_Width) / float(u_PixelSize))
        , float(pixelNumY) / (float(u_Height) / float(u_PixelSize)));

    float lastBlurCoof = texture(u_Noise, currentPixel).r;

    if (lastBlurCoof < MAX_COOF) {
        color2  = blur5(vec2(float(u_WindPowerX) * coofXB, float(u_WindPowerY) * coofYB),
         vec2(float(u_Width), float(u_Height)), curColor);
    } else if (u_Type == 1 || lastBlurCoof < 2. * MAX_COOF) {
        color2 = blur5(vec2(float(u_WindPowerX) * coofXB * COOF_SMALL_SHIFT,
                          float(u_WindPowerY) * coofYB * COOF_SMALL_SHIFT),
                          vec2(float(u_Width), float(u_Height)), curColor);

        color2 = mix(curColor, color2, color2.a);

        color1 = blur5(2. * vec2(float(u_WindPowerX) * coofXB * COOF_SMALL_SHIFT,
                                  2. * float(u_WindPowerY) * coofYB * COOF_SMALL_SHIFT),
                                  vec2(float(u_Width), float(u_Height)), curColor);

        color2 = mix(color1, color2, color2.a);
    } else {
        color2 = curColor;
    }

    o_FragColor = color2;
}

